// *************************************************************************
// Include: loc_sock                  Versin : 1.1.0     FECHA : 14/jul/00
//
// Descripcin :
//   Funciones bsicas de trabajo con Local Sockets (AF_UNIX) tipo datagrama
//   Permiten definir, enviar y recibir datos a travs de Unix Sockets
//
// Autor : Antonio E. Martnez                            Fecha : 14/jul/00
// Centro: UAM - II
//
// Cambios:
// *************************************************************************

#ifndef LOC_SOCKH
#define LOC_SOCKH

#include <stdio.h>
#include <sys/socket.h>
#include <sys/un.h>

#define LONGMAXPATH 108

int loc_socket(void);
// *************************************************************************
// Funcin : loc_socket
//
// Descripcin: Adquiere un socket local tipo datagram
//
// Cdigo de retorno:
//   >0: Identificador del socket que se adquiere.
//   -1: Ha habido algn error al adquirir el socket. Se ha enviado un mensaje
//       de error a stderr.
// *************************************************************************

int loc_bind(int s_id, char * path);
// *************************************************************************
// Funcin : loc_bind
//
// Descripcin: Enlaza el socket S con el fichero cuyo path se proporciona 
//   en el segundo parmetro, para recibir informacin por l de cualquier 
//   aplicacin local.
//
// Parmetros:
//    int s_id: Identificador del socket a enlazar
//    char * path: Path empleado para enlazar el socket.
//
// Cdigo de retorno:
//    0: Operacin realizada satisfactoriamente.
//   -1: Error en el enlace. Se ha enviado un mensaje de error a stderr.
// *************************************************************************

int loc_sendto(int s_id, void * mensaje, int longitud, char *destino);
// *************************************************************************
// Funcin : loc_sendto
//
// Descripcin: Enva los datos contenidos en el parmetro mensaje, cuyo
//   tamao se encuentra en el parmetro longitud, a travs del socket de
//   identificador s_id, a la direccin especificada en Destino
//
// Parmetros:
//    int s_id; Identificador del socket.
//    void * mensaje: Puntero a los datos a transmitir.
//    int longitud: Longitud de datos a transmitir.
//    char * destino: Path del fichero enlazado al socket destino.
//
// Cdigo de retorno:
//    >0: Nmero total de bytes transmitidos.
//    -1: Error en el envo. Se ha enviado un mensaje de error a stderr.
// *************************************************************************

int loc_recvfrom(int s_id, void * mensaje, int longitud, char * origen);
// *************************************************************************
// Funcin : loc_recvfrom
//
// Descripcin: Recibe datos a travs del socket s_id y los almacena en la 
//   zona de memoria apuntada por mensaje.
//
// Parmetros:
//    int s_id: Identificador del socket.
//    void * mensaje: Buffer donde almacenar el mensaje recibido.
//    int longitud: Longitud mxima del buffer. Si el mensaje recibido es
//      de mayor longitud, se trunca y se pierde el resto.
//    char * origen: Path del fichero enlazado al socket origen de la 
//      comunicacin.
//
// Cdigo de retorno:
//    >0: Nmero total de bytes transmitidos.
//     0: Conexin cerrada;
//    -1: Error en la recepcin. Se ha enviado un mensaje de error a stderr.
//    Salida :
// *************************************************************************

int loc_soclose(int s_id);
// *************************************************************************
// Funcin : loc_soclose
//
// Descripcin: Cierra una comunicacion por el socket s.
//
// Parmetros:
//    int s_id: Identificador del socket
//
// Cdigo de retorno:
//    0: Socket cerrado satisfactoriamente.
//    -1: Error en el cierre. Se ha enviado un mensaje de error a stderr.
// *************************************************************************

int loc_trazas(const char * fichero);
// *************************************************************************
// Funcin : loc_trazas
//
// Descripcin: Establece un fichero de trazas para depuracin
//
// Parmetros:
//    const char * fichero: Nombre del fichero de trazas que se desea.
//	Si es NULL, se cierran las trazas.
// Cdigo de retorno:
//    0: Trazas establecidas correctamente.
//   -1: Imposible abrir fichero de trazas.
// *************************************************************************
#endif
