/*************************************************************************
 * Include : nivel1ppp.h              Versin : 2.0.0     FECHA : 20/sep/02
 *
 * Descripcin :
 *   Funciones de interfaz con Level 1 PPP para prcticas de
 *	Redes de Comunicaciones
 *
 * Autor : Antonio E. Martnez                            Fecha : 16/oct/01
 * Centro: UAM - II
 *
 * Cambios:
 *   * 20/09/2002. V 2.0.0. Antonio Martnez
 *      - Cambiados los nombres de las funciones de la interfaz
 *************************************************************************/

#ifndef _nivel1PPP_H_
#define _nivel1PPP_H_

#include <termios.h>	/* Definiciones de velocidades transmisin       */

/**************************** Tipos de datos  ****************************/

#ifndef _BYTE_
#define _BYTE_
typedef unsigned char BYTE;
#endif

#ifndef _WORD_
#define _WORD_
typedef unsigned short WORD;
#endif

/* Tipo de datos de Puntero a la funcin de notificacin desde nivel 1   */
typedef int (*tpfNotificacionDesdeNivel1) (int, const BYTE *);

/************************ Definicin de constantes ***********************/

/* Tamaos tramas sin secuencias de escape                               */
#define ETH_TLEN      2     	   /* Tamao del campo ethertype         */
#define PPP_CLEN      2            /* Longitud campos direccin y control*/
#define PPP_HLEN      4            /* Cabecera trama PPP                 */
#define PPP_FRAME_MAX 1514  	   /* Tamao mximo de trama PPP         */
#define PPP_DATA_MAX  (PPP_FRAME_MAX - 4) /* Tamao mximo datos         */

/* En codificacin de lnea (considerando secuencias de escape)          */
#define PPP_LIN_HLEN      3        /* Cabecera trama PPP: FF 7D 03       */
#define PPP_LIN_FRAME_MAX PPP_FRAME_MAX*2 /* Tamao maximo trama PPP     */
#define PPP_LIN_DATA_MAX  PPP_DATA_MAX*2  /* Tamao mximo de datos      */

/* Puertos de comunicaciones                                             */
#define COM1 "/dev/ttyS0"
#define COM2 "/dev/ttyS1"

/* Flag delimitadora de tramas PPP                                       */
#define PPP_FLAG 0x7E

/* Cdigos de retorno de las funciones                                   */
#define ETH_OK        0
#define ETH_ERROR     -1
#define ETH_TIMEOUT   -2


/************************* prototipos de funciones ***********************/

/*************************************************************************
 * Funcion: EsperaNuevaTrama
 * Fecha: 24.05.2000
 * Autor: Antonio Martnez
 *
 * Espera a que est disponible una nueva trama para su lectura
 *
 * Entrada:
 *    int timeout: Valor en ms de tiempo transcurrido el cual se devuelve
 *	control aunque no haya llegado ninguna trama.
 *	0 o negativo si no se desea timeout.
 * Salida:
 *   ETH_OK: Trama recibida
 *   ETH_TIMEOUT: Salida por timeout
 *************************************************************************/
int EsperaNuevaTrama(int t);

/*************************************************************************
 * Funcion: IniciarNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que inicializa Nivel1.
 *
 * Entrada:
 *    int num_types: numero de tipos ethernet en ether_types (0-2)
 *    const BYTE *ether_types: tipos de tramas ethernet que se quiere
 *                             recibir
 *    tpfNotifcacionDesdeNivel1 interrupt_func:
 *         funcion de interrupcion que se llamara cada vez que se reciba
 *         una trama (puede ser NULL si nprots == 0)
 *    int timer_intr: Solicitud de llamada a la funcin de
 *	   interrupcin por timer cada timer_intr milisegundos.
 *	   Si se pasa valor 0 o negativo no se producen llamadas por timer.
 *    int port: Puerto serie por el que se realiza la transmisin. Puede valer
 *         1 (COM1) o 2 (COM2)
 *    speed_t speed: Velocidad del puerto. Acepta los valores definidos en
 *         bits/termios.h, que expresan la velocidad en bits/s:
 *           B50	B134	B300	B1800	B9600
 *           B75	B150	B600	B2400	B19200
 *           B110	B200	B1200	B4800	B38400
 *
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int IniciarNivel1(int num_types, const BYTE *ether_types,
                  tpfNotificacionDesdeNivel1 interrupt_func,
                  int timer_intr, int port, speed_t speed);

/*************************************************************************
 * Funcion: FinalizarNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que finaliza Nivel1 y libera todos los recursos que ocupa.
 *
 * Entrada:
 *    ninguna
 * Salida:
 *    0 si todo ha ido bien y -1 en caso de error
 *************************************************************************/
int FinalizarNivel1();

/*************************************************************************
 * Funcion: EnviarTramaNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que envia una trama ethernet.
 *
 * Entrada:
 *    int len: Longitud de la trama
 *    const BYTE * data: Trama a transmitir
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int EnviarTramaNivel1(int len, const BYTE *data);

/*************************************************************************
 * Funcion: RecibirTramaNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que recibe una trama. Solo se debe llamar cuando
 * nueva_trama = 1.
 *
 * Entrada:
 *    BYTE *buffer: puntero donde se almacenara la trama recibida
 * Salida:
 *   Numero de bytes recibidos si todo ha ido bien
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int RecibirTramaNivel1(BYTE *buffer);

/*************************************************************************
 * Funcion: ActivarTrazas
 * Fecha: 14.07.2000
 * Autor: Antonio Martnez
 *
 * Funcion que activa el modo debug, produciendo trazas de la operacin
 *
 * Entrada:
 *    int tipo: Tipo de trazas que se pide.
 *      0: Suprimir las trazas.
 *      1: Trazas de tramas que se envan y reciben.
 *      2: Trazas nivel 1, y de llamadas a funciones de Nivel1.
 *      3: Trazas nivel 2, y de comunicacin con ethd.
 *    char *fichero: Nombre del fichero donde se realizan las trazas.
 *	Acepta stdout y stderr como posibles valores
 * Salida:
 *    0 si todo ha ido bien y -1 en caso de error
 *************************************************************************/
int ActivarTrazas(int tipo, char * fichero);

#endif /* _Nivel1PPP_H_ */
